# -*- coding: utf-8 -*-
#
# Copyright (C) 2003-2009 Edgewall Software
# Copyright (C) 2003-2004 Jonas Borgström <jonas@edgewall.com>
# Copyright (C) 2004-2005 Christopher Lenz <cmlenz@gmx.de>
# All rights reserved.
#
# This software is licensed as described in the file COPYING.trac, which
# you should have received as part of this distribution. The terms
# are also available at http://trac.edgewall.org/wiki/TracLicense.
#
# This software consists of voluntary contributions made by many
# individuals. For the exact contribution history, see the revision
# history and logs, available at http://trac.edgewall.org/log/.
#
# Author: Luke Tucker <voxluci@gmail.com>
#         Jonas Borgström <jonas@edgewall.com>
#         Christopher Lenz <cmlenz@gmx.de>

from glob import glob
import imp
import os
import sys
import traceback

__all__ = ['find_plugins_in_path', 'find_plugins_by_entry_point']

import logging 
log = logging.getLogger(__name__)

def find_plugins_in_path(search_path):
    """
    Discover plugins in any .py files in the given on-disk locations eg:
    
    find_plugins_in_path("/path/to/mymodule/plugins")
    find_plugins_in_path(["/path/to/mymodule/plugins", "/some/more/plugins"])
    
    """
    if isinstance(search_path, basestring):
        search_path = [search_path]

    for path in search_path:
        log.debug("searching for plugins in %s" % search_path)
        for py_file in glob(os.path.join(path, '*.py')):
            try:
                module_name = os.path.basename(py_file[:-3])
                # if it's already loaded, move on 
                if module_name in sys.modules:
                    continue
                
                log.debug("Loading module %s" % py_file)
                module = imp.load_source(module_name, py_file)
            except:
                log.error("Error loading module %s: %s" % (os.path.join(path, py_file), traceback.format_exc()))



try:
    from pkg_resources import working_set as master_working_set

    def find_plugins_by_entry_point(entry_point_id, ws=master_working_set):
        for entry in ws.iter_entry_points(entry_point_id):
            log.debug('Loading plugin %s from %s', entry.name, entry.dist.location)

            try:
                entry.load(require=True)
            except:
                import traceback
                log.error("Error loading plugin %s from %s: %s" % (entry.name, entry.dist.location, traceback.format_exc()))
        
except ImportError:
    
    def find_plugins_by_entry_point(entry_point_id, ws=None):
        log.warning("Not loading plugins from eggs, setuptools not found.")


# -*- coding: utf-8 -*-
#
# Copyright (C) 2003-2009 Edgewall Software
# Copyright (C) 2003-2004 Jonas Borgström <jonas@edgewall.com>
# Copyright (C) 2004-2005 Christopher Lenz <cmlenz@gmx.de>
# All rights reserved.
#
# This software is licensed as described in the file COPYING.trac, which
# you should have received as part of this distribution. The terms
# are also available at http://trac.edgewall.org/wiki/TracLicense.
#
# This software consists of voluntary contributions made by many
# individuals. For the exact contribution history, see the revision
# history and logs, available at http://trac.edgewall.org/log/.
#
# Author: Luke Tucker <voxluci@gmail.com>
#         Jonas Borgström <jonas@edgewall.com>
#         Christopher Lenz <cmlenz@gmx.de>


from zope import interface as zi
from zope.interface import Attribute, Interface
from zope.interface.advice import addClassAdvisor

__all__ = ['Attribute', 'Component', 'ComponentManager', 'ExtensionPoint', 'ExtensionInterface', 
           'implements', 'implements_only', 'implemented_by', 'is_implemented_by', 
           'ExtensionError']

class ExtensionError(Exception):
    """Exception for extension related errors."""

class ExtensionInterface(Interface):
    """Marker class for extension interfaces and extensions"""

class ExtensionPoint(property):
    """Marker class for extension points in components."""

    def __init__(self, interface):
        """Create the extension point.
        
        @param interface: the `ExtensionInterface` subclass that defines the protocol
            for the extension point
        """
        property.__init__(self, self.extensions)
        self.interface = interface
        self.__doc__ = 'List of components that implement `%s`' % \
                       self.interface.__name__

    def extensions(self, component):
        """Return a list of components that declare they implement the extension
        point interface.
        """
        return component.compmgr.get_all(self.interface)

    def __repr__(self):
        """Return a textual representation of the extension point."""
        return '<ExtensionPoint %s>' % self.interface.__name__


class ComponentMeta(type):
    """Meta class for components.
    
    Takes care of component and extension point registration.
    """
    _components = []
    _registry = {}

    def __new__(cls, name, bases, d):
        """Create the component class."""

        new_class = type.__new__(cls, name, bases, d)
        if name == 'Component':
            # Don't put the Component base class in the registry
            return new_class

        # Only override __init__ for Components not inheriting ComponentManager
        if not any(issubclass(x, ComponentManager) for x in bases):
            # Allow components to have a no-argument initializer so that
            # they don't need to worry about accepting the component manager
            # as argument and invoking the super-class initializer
            init = d.get('__init__')
            if not init:
                # Because we're replacing the initializer, we need to make sure
                # that any inherited initializers are also called.
                for init in [b.__init__._original for b in new_class.mro()
                             if issubclass(b, Component)
                             and '__init__' in b.__dict__]:
                    break
            def maybe_init(self, compmgr, init=init, cls=new_class):
                component_id = _component_id(cls)
                if component_id not in compmgr.components:
                    compmgr.components[component_id] = self
                    if init:
                        try:
                            init(self)
                        except:
                            del compmgr.components[component_id]
                            raise
            maybe_init._original = init
            new_class.__init__ = maybe_init

        if d.get('abstract'):
            # Don't put abstract component classes in the registry
            return new_class

        ComponentMeta._components.append(new_class)
        
        # if there are interfaces implemented by this class, 
        # a class adivsor calling _register_interfaces
        # will be called when zope.interface has finished 
        # working on it. Otherwise, we just register now
        # to pick up any inherited interfaces. 
        if new_class.__dict__.get('__implements_advice_data__'):
            return new_class
        else:
            return _register_interfaces(new_class)
            
def _register_interfaces(cls):
    # skip classes that were not determined to be 
    # concrete Components by the component metaclass.
    
    if not cls in ComponentMeta._components:
        return cls
        
    registry = ComponentMeta._registry
    for interface in zi.implementedBy(cls).__iro__:
        if interface.extends(ExtensionInterface):
            registry.setdefault(interface, [])
            if not cls in registry[interface]:
                registry[interface].append(cls)
                
    return cls

##############################################################################
# XXX monkey patch for mixing with zope.interface
# this monkey patch allows us to attach an additional class advisor that
# is executed after the class advisor generated by any of the implements()
# type declarations. The class advisor allows us to register the interfaces
# that are set up by the zope.interface class advisor.
import sys
from zope.interface import declarations
from zope.interface.declarations import _implements_advice
def _patched_implements(name, interfaces, classImplements):
    frame = sys._getframe(2)
    locals = frame.f_locals

    # Try to make sure we were called from a class def. In 2.2.0 we can't
    # check for __module__ since it doesn't seem to be added to the locals
    # until later on.
    if (locals is frame.f_globals) or (
        ('__module__' not in locals) and sys.version_info[:3] > (2, 2, 0)):
        raise TypeError(name+" can be used only from a class definition.")

    if '__implements_advice_data__' in locals:
        raise TypeError(name+" can be used only once in a class definition.")

    locals['__implements_advice_data__'] = interfaces, classImplements
    addClassAdvisor(_implements_advice, depth=3)
    addClassAdvisor(_register_interfaces, depth=3)
declarations._implements = _patched_implements
###############################################################################

def implements(*interfaces):
    """
    Declare a list of ExtensionInterfaces implemented by a 
    Component.
    """
    _patched_implements("implements", interfaces, zi.classImplements)
    

def implements_only(*interfaces):
    """
    declare that a Component implements exactly the list 
    of ExtensionInterfaces specified, overriding any 
    inherited declarations of implemented ExtensionInterfaces.
    """
    _patched_implements("implementsOnly", interfaces, zi.classImplementsOnly)

def implemented_by(thang):
    """
    return the list of ExtensionInterfaces implemented by a Component
    or Component type.
    """
    if isinstance(thang, type):
        get_ifaces = zi.implementedBy
    else: 
        get_ifaces = zi.providedBy

    ifaces = []
    for cls in get_ifaces(thang).__iro__:
        if hasattr(cls, 'extends') and cls.extends(ExtensionInterface):
            ifaces.append(cls)
    return ifaces
    
def is_implemented_by(cls, iface):
    """
    returns True if cls implements the interface given
    """
    return iface in implemented_by(cls)
    
def _component_id(component):
    if isinstance(component, basestring):
        return component

    if not isinstance(component, type):
        component = component.__class__

    return "%s.%s" % (component.__module__, component.__name__)

class Component(object):
    """Base class for components.

    Every component can declare what extension points it provides, as well as
    what extension points of other components it extends.
    """
    __metaclass__ = ComponentMeta

    def __new__(cls, *args, **kwargs):
        """Return an existing instance of the component if it has already been
        activated, otherwise create a new instance.
        """
        # If this component is also the component manager, just invoke that
        if issubclass(cls, ComponentManager):
            self = super(Component, cls).__new__(cls)
            self.compmgr = self
            return self

        # The normal case where the component is not also the component manager
        compmgr = args[0]
        component_id = _component_id(cls)
        self = compmgr.components.get(component_id)
        if self is None:
            self = super(Component, cls).__new__(cls)
            self.compmgr = compmgr
            compmgr.component_activated(self)
        return self


class ComponentManager(object):
    """The component manager keeps a pool of active components."""

    def __init__(self):
        """Initialize the component manager."""
        self.components = {}
        if isinstance(self, Component):
            my_id = _component_id(self)
            self.components[my_id] = self
        self._restriction = None

    def __contains__(self, cls):
        """Return wether the given class is in the list of active components."""
        return _component_id(cls) in self.components

    def get_all(self, iface):
        """
        retrieves implementors of the interface specified.
        """
        return filter(None, [self._get_instance_of(cls) for cls in
                      ComponentMeta._registry.get(iface, [])])

    def _get_instance_of(self, cls):
        """Activate the component instance for the given class, or return the
        existing the instance if the component has already been activated.
        """
        if not self.is_component_enabled(cls):
            return None

        component_id = _component_id(cls)
        component = self.components.get(component_id)
        if component is None:
            if cls not in ComponentMeta._components:
                raise ExtensionError('Component "%s" not registered' % cls.__name__)
            try:
                component = cls(self)
            except TypeError, e:
                raise ExtensionError('Unable to instantiate component %r (%s)' %
                                (cls, e))
        return component

    def component_activated(self, component):
        """Can be overridden by sub-classes so that special initialization for
        components can be provided.
        """
    
    def restrict(self, policy):
        """
        restrict enabled components according to the policy 
        given -- is_component_enabled will be delegated to the 
        object specified.
        """
        self._restriction = policy

    def is_component_enabled(self, cls):
        """Controlled by policy given at construction time, but can be overridden 
        by sub-classes.

        If this method returns False, the component with the given class will
        not be available.
        
        By default, this method returns True
        """
        if self._restriction is None:
            return True
        else:
            return self._restriction.is_component_enabled(cls)


def test_docs():
    from doctest import testfile
    import os
    docdir = '../doc'
    total_failures = 0
    total_tests = 0
    for root, dirs, files in os.walk(docdir):
        for fn in files:
            if fn.endswith('.rst'):
                docfile = os.path.join(root, fn)
                fail, nt = testfile(docfile)
                total_failures += fail
                total_tests += nt
    assert total_failures == 0
        


# -*- coding: utf-8 -*-
#
# Copyright (C) 2009-2010 Luke Tucker
# All rights reserved.
#
# This software is licensed as described in the file COPYING, which
# you should have received as part of this distribution.
#
# Author: Luke Tucker <voxluci@gmail.com>
#

from giblets.core import *


import fnmatch
import re

from giblets.core import _component_id

__all__ = ['Blacklist', 'Whitelist', 'Patterns']

class Blacklist(object):
    """
    policy that activates all Components that
    have not been specifically disabled.
    """

    def __init__(self):
        self.blacklist = set()

    def enable_component(self, component):
        """
        Allow the component specified to be activated
        for this ComponentManager.

        component may be a full class name string 'foo.bar.Quux' 
        a Component type or an instance of a Component.
        """
        try:
            self.blacklist.remove(_component_id(component))
        except KeyError:
            pass

    def disable_component(self, component):
        """
        Do not allow the component specified to be activated
        for this ComponentManager.

        component may be a full class name string 'foo.bar.Quux' 
        a Component type or an instance of a Component.
        """
        component_id = _component_id(component)
        self.blacklist.add(component_id)

    def is_component_enabled(self, component):
        """
        returns True unless the component specified 
        has been disabled.
        """
        if _component_id(component) in self.blacklist:
            return False
        return True

class Whitelist(object):
    """
    Policy that activates only Components
    that have been specifically enabled.
    """

    def __init__(self):
        self.whitelist = set()

    def enable_component(self, component):
        """
        Allow the component specified to be activated
        for this ComponentManager.

        component may be a full class name string 'foo.bar.Quux' 
        a Component type or an instance of a Component.
        """
        self.whitelist.add(_component_id(component))

    def disable_component(self, component):
        """
        Do not allow the component specified to be activated
        for this ComponentManager.

        component may be a full class name string 'foo.bar.Quux' 
        a Component type or an instance of a Component.
        """
        try:
            self.whitelist.remove(_component_id(component))
        except KeyError:
            pass

    def is_component_enabled(self, component):
        """
        returns False unless the component specified has been 
        enabled.
        """
        return _component_id(component) in self.whitelist

class Patterns(object):
    """
    A policy which enables and disables components
    based on an ordered list of wildcard patterns like foo.bar.* 
    etc. First match is taken. 
    """
    
    def __init__(self):
        self.patterns = []

    def build_pattern(self, pattern, enable):
        """
        create an entry suitable for insertion into the 
        patterns list of this manager. 
        
        If pattern is a string, it is 
        treated as a wildcard patten like foo.bar.*
        
        Otherwise, it is assumed pattern is a compiled
        regular expression.
        
        eg: 
        pat = mgr.build_pattern('foo.*', True)
        mgr.patterns.insert(0, pat)
        """
        if isinstance(pattern, basestring):
            pattern = re.compile(fnmatch.translate(pattern))
        return (pattern, enable)

    def append_pattern(self, pattern, enable):
        """
        helper to add a pattern to the end of the 
        list of patterns.
        """
        pat = self.build_pattern(pattern, enable)
        self.patterns.append(pat)

    def is_component_enabled(self, component):
        enabled = False
        comp_id = _component_id(component)
        for (pat, state) in self.patterns:
            if pat.match(comp_id) is not None:
                enabled = state
                break
        return enabled




# -*- coding: utf-8 -*-
#
# Copyright (C) 2009-2010 Luke Tucker
# All rights reserved.
#
# This software is licensed as described in the file COPYING, which
# you should have received as part of this distribution.
#
# Author: Luke Tucker <voxluci@gmail.com>
#
from helpers import *

def test_implemented_by():
    clear_registry()
    from giblets import Component, ComponentManager, ExtensionPoint
    from giblets import ExtensionInterface, implements, implemented_by

    class IFoo(ExtensionInterface):
        pass
        
    class IBar(ExtensionInterface):
        pass
        
    class Foo(Component):
        implements(IFoo)
    
    class Bar(Component):
        implements(IBar)
        
    mgr = ComponentManager()
    foo = Foo(mgr)
    bar = Bar(mgr)
    
    assert IFoo in implemented_by(Foo)
    assert IFoo in implemented_by(foo)
    assert len(list(implemented_by(Foo))) == len(list(implemented_by(foo))) == 1
        
    assert IBar in implemented_by(Bar)
    assert IBar in implemented_by(bar)
    assert len(list(implemented_by(Bar))) == len(list(implemented_by(bar))) == 1        

def test_is_implemented_by():
    from giblets import Component, ComponentManager, ExtensionPoint
    from giblets import ExtensionInterface, implements, is_implemented_by

    class IFoo(ExtensionInterface):
        pass
        
    class IBar(ExtensionInterface):
        pass
        
    class Foo(Component):
        implements(IFoo)
    
    class Bar(Component):
        implements(IBar)
        
    mgr = ComponentManager()
    foo = Foo(mgr)
    bar = Bar(mgr)
    
    assert is_implemented_by(Foo, IFoo)
    assert is_implemented_by(foo, IFoo)
    assert not is_implemented_by(Foo, IBar)
    assert not is_implemented_by(foo, IBar)
    
    assert is_implemented_by(Bar, IBar)
    assert is_implemented_by(bar, IBar)
    assert not is_implemented_by(Bar, IFoo)
    assert not is_implemented_by(bar, IFoo)

def test_is_implemented_by_implements_only():
    
    from giblets import Component, ComponentManager, ExtensionPoint
    from giblets import ExtensionInterface, implements, is_implemented_by
    from giblets import implements_only

    class IVehicle(ExtensionInterface):
        pass

    class IFloat(ExtensionInterface):
        pass

    class IBoat(IVehicle):
        pass

    class Sailboat(Component):
        implements(IBoat, IFloat)

    mgr = ComponentManager()

    sailboat = Sailboat(mgr)
    assert is_implemented_by(Sailboat, IBoat)
    assert is_implemented_by(sailboat, IBoat)
    assert is_implemented_by(Sailboat, IFloat)
    assert is_implemented_by(sailboat, IFloat)
    assert is_implemented_by(Sailboat, IVehicle)
    assert is_implemented_by(sailboat, IVehicle)


    class Sloop(Sailboat):
       pass

    sloop = Sloop(mgr)
    assert is_implemented_by(Sloop, IBoat)
    assert is_implemented_by(sloop, IBoat)
    assert is_implemented_by(Sloop, IFloat)
    assert is_implemented_by(sloop, IFloat)
    assert is_implemented_by(Sloop, IVehicle)
    assert is_implemented_by(sloop, IVehicle)

    class BoatWithAHole(Sailboat):
        implements_only(IBoat)

    bwah = BoatWithAHole
    assert is_implemented_by(BoatWithAHole, IBoat)
    assert is_implemented_by(bwah, IBoat)
    assert not is_implemented_by(BoatWithAHole, IFloat)
    assert not is_implemented_by(bwah, IFloat)
    assert is_implemented_by(BoatWithAHole, IVehicle)
    assert is_implemented_by(bwah, IVehicle)


def test_extension_reg():
    clear_registry()
    from giblets import Component, ComponentManager, ExtensionPoint, ExtensionInterface, implements


    # a Train is extended by a bunch of train cars...
    class ITrainCar(ExtensionInterface):
        pass

    class Train(Component):
        cars = ExtensionPoint(ITrainCar)

    class EngineCar(Component):
        implements(ITrainCar)
        
    class PassengerCar(Component):
        implements(ITrainCar)
        
    class CafeCar(Component):
        implements(ITrainCar)
    
    class CabooseCar(Component):
        implements(ITrainCar)
        

    # an irrelevant interface and implementation...
    class IZooAnimal(ExtensionInterface):
        pass
        
    class Zebra(Component):
        implements(IZooAnimal)

    # create a component manager and the train associated with it 
    mgr1 = ComponentManager()
    train1 = Train(mgr1)
    
    # components are singletons w/ respect to manager
    assert id(train1) == id(Train(mgr1))
    
    # check that all of the extensions for the extension point were registered
    for cls in (EngineCar, PassengerCar, CafeCar, CabooseCar):
        assert has_exactly(1, cls, train1.cars)
        assert has_exactly(1, cls, mgr1.get_all(ITrainCar))
    
    # check that the Zebra is not a part of the train
    assert has_exactly(0, Zebra, train1.cars)
    assert has_exactly(0, Zebra, mgr1.get_all(ITrainCar))
    
    # create a different component manager and the train associated with it, 
    # check the same stuff...
    mgr2 = ComponentManager()
    train2 = Train(mgr2)
    assert id(train2) == id(Train(mgr2))
    for cls in (EngineCar, PassengerCar, CafeCar, CabooseCar):
        assert has_exactly(1, cls, train2.cars)
        assert has_exactly(1, cls, mgr2.get_all(ITrainCar))

    assert has_exactly(0, Zebra, train2.cars)
    assert has_exactly(0, Zebra, mgr2.get_all(ITrainCar))

    # now check that the trains and extensions produced by the
    # different component managers are different.
    assert train1 != train2
    for cls in (EngineCar, PassengerCar, CafeCar, CabooseCar):
        i1 = (x for x in train1.cars if isinstance(x, cls)).next()
        i2 = (x for x in train2.cars if isinstance(x, cls)).next()
        assert id(i1) != id(i2)
        
        i1 = (x for x in mgr1.get_all(ITrainCar) if isinstance(x, cls)).next()
        i2 = (x for x in mgr2.get_all(ITrainCar) if isinstance(x, cls)).next()

        assert id(i1) != id(i2)
        
        
        
def test_multi_extension():
    clear_registry()
    from giblets import Component, ComponentManager, ExtensionPoint, ExtensionInterface, implements
    
    # set up some scenarios that test components providing multiple 
    # extensions and extension interfaces that imply other 
    # extension interfaces by inheritance
        
    # a parking lot gets all wheeled vehicles
    class IWheeledVehicle(ExtensionInterface):
        pass
    class ParkingLot(Component):
        vehicles = ExtensionPoint(IWheeledVehicle)

    # an airport gets all flying vehicles
    class IFlyingVehicle(ExtensionInterface):
        pass
    class Airport(Component):
        vehicles = ExtensionPoint(IFlyingVehicle)
    
    # a flying parking lot is only populated by flying cars, 
    # which also happen to be wheeled and flying
    class IFlyingCar(IWheeledVehicle, IFlyingVehicle):
        pass
    class FlyingParkingLot(Component):
        vehicles = ExtensionPoint(IFlyingCar)

    # now some vehicles implementing the various types...    
    class Sedan(Component):
        """
        This class is only a wheeled vehicle
        """
        implements(IWheeledVehicle)

    class Helicopter(Component):
        """
        This class is only a flying vehicle
        """
        implements(IFlyingVehicle)
        
    class FlyingBaloneyTruck(Component):
        """
        This class is a flying car, it is also 
        therefore wheeled and flying because the 
        interface inherits from both.
        """
        implements(IFlyingCar)


    class WheeledBalloon(Component):
        """
        This class is wheeled and flying, but is 
        not a flying car. 
        """
        implements(IWheeledVehicle, IFlyingVehicle)

    mgr = ComponentManager()
    
    # check that wheeled vehicles are in the parking lot
    parking_lot = ParkingLot(mgr)
    assert has_exactly(1, Sedan, parking_lot.vehicles)
    assert has_exactly(0, Helicopter, parking_lot.vehicles)
    assert has_exactly(1, FlyingBaloneyTruck, parking_lot.vehicles)
    assert has_exactly(1, WheeledBalloon, parking_lot.vehicles)
    
    # check that flying vehicles are in the airport
    airport = Airport(mgr)
    assert has_exactly(0, Sedan, airport.vehicles)
    assert has_exactly(1, Helicopter, airport.vehicles)
    assert has_exactly(1, FlyingBaloneyTruck, airport.vehicles)
    assert has_exactly(1, WheeledBalloon, airport.vehicles)
    
    # check that flying cars are in the flying parking lot
    flying_lot = FlyingParkingLot(mgr)
    assert has_exactly(0, Sedan, flying_lot.vehicles)
    assert has_exactly(0, Helicopter, flying_lot.vehicles)
    assert has_exactly(1, FlyingBaloneyTruck, flying_lot.vehicles)
    assert has_exactly(0, WheeledBalloon, flying_lot.vehicles)
    
    

def test_custom_mgr():
    clear_registry()
    from giblets import Component, ComponentManager, ExtensionPoint, ExtensionInterface, implements

    class ICog(ExtensionInterface):
        pass

    class Widget(Component):
        cogs = ExtensionPoint(ICog)

    class GoodCog(Component):
        implements(ICog)

    class BadCog(Component):
        implements(ICog)
    
    # custom component manager
    class FilterComponentManager(ComponentManager):
        def is_component_enabled(self, cls):
            return cls != BadCog
    
    mgr = FilterComponentManager()
    widget = Widget(mgr)
    assert has_exactly(1, GoodCog, widget.cogs)
    assert has_exactly(0, BadCog, widget.cogs)

def test_inherited_interfaces():
    clear_registry()
    from giblets import Component, ComponentManager, ExtensionPoint, ExtensionInterface, implements

    class IThing1(ExtensionInterface):
        pass

    class IThing2(ExtensionInterface):
        pass

    class IThing3(ExtensionInterface):
        pass

    class IThing4(ExtensionInterface):
        pass
        
    class Thing1(Component):
        implements(IThing1)
    
    class Thing2(Component):
        implements(IThing2)
    
    class Thing3(Thing1, Thing2):
        implements(IThing3)
        
    class Thing4(Thing3):
        implements(IThing4)
        
    class ThingSorter(Component):
        thing1 = ExtensionPoint(IThing1)
        thing2 = ExtensionPoint(IThing2)
        thing3 = ExtensionPoint(IThing3)
        thing4 = ExtensionPoint(IThing4)
        
    mgr = ComponentManager()
    sorter = ThingSorter(mgr)
    
    assert len(sorter.thing1) == 3
    assert has_exactly(3, Thing1, sorter.thing1)
    assert has_exactly(2, Thing3, sorter.thing1)
    assert has_exactly(1, Thing4, sorter.thing1)
    
    assert len(sorter.thing2) == 3
    assert has_exactly(3, Thing2, sorter.thing2)
    assert has_exactly(2, Thing3, sorter.thing2)
    assert has_exactly(1, Thing4, sorter.thing2)
    
    assert len(sorter.thing3) == 2
    assert has_exactly(2, Thing3, sorter.thing3)
    assert has_exactly(1, Thing4, sorter.thing3)
    
    assert len(sorter.thing4) == 1
    assert has_exactly(1, Thing4, sorter.thing4)
    
    
    
def test_abstract_component():
    clear_registry()
    from giblets import Component, ComponentManager, ExtensionPoint, ExtensionInterface, implements
    
    class ISomething(ExtensionInterface):
        pass

    class BaseThing(Component):
        implements(ISomething)
        abstract = True

    class ConcreteThing(BaseThing):
        pass

    class ThingBox(Component):
        things = ExtensionPoint(ISomething)

    mgr = ComponentManager()
    
    thingbox = ThingBox(mgr)

    assert len(thingbox.things) == 1
    assert has_exactly(1, ConcreteThing, thingbox.things)
    
def test_implements_only():
    clear_registry()
    from giblets import Component, ComponentManager, ExtensionPoint, ExtensionInterface, implements, implements_only

    class ICog(ExtensionInterface):
        pass
        
    class IWidget(ExtensionInterface):
        pass
        
    class Cog(Component):
        implements(ICog)
        
    class CogWidget(Cog):
        implements(IWidget)
        
    class NoCogWidget(Cog):
        implements_only(IWidget)
        
    class Machine(Component):
        cogs = ExtensionPoint(ICog)
        widgets = ExtensionPoint(IWidget)
        
    mgr = ComponentManager()
    machine = Machine(mgr)
    
    assert len(machine.cogs) == 2
    assert has_exactly(2, Cog, machine.cogs)
    assert has_exactly(1, CogWidget, machine.cogs)
    assert has_exactly(0, NoCogWidget, machine.cogs)
    
    assert len(machine.widgets) == 2
    assert has_exactly(1, CogWidget, machine.widgets)
    assert has_exactly(1, NoCogWidget, machine.widgets)



from helpers import *

def test_blacklist():
    clear_registry()
    from giblets import Component, ComponentManager, ExtensionPoint, ExtensionInterface, implements
    from giblets.policy import Blacklist

    class ICog(ExtensionInterface):
        pass
        
    class Widget(Component):
        cogs = ExtensionPoint(ICog)
     
    class GoodCog(Component):
        implements(ICog)
        
    class BadCog(Component):
        implements(ICog)
        
    # first no restrtictions, both should show up
    policy = Blacklist()
    
    mgr = ComponentManager()
    mgr.restrict(policy)
    widget = Widget(mgr)
    assert has_exactly(1, GoodCog, widget.cogs)
    assert has_exactly(1, BadCog, widget.cogs)
    
    policy.disable_component(BadCog)
    assert has_exactly(1, GoodCog, widget.cogs)
    assert has_exactly(0, BadCog, widget.cogs)

    # re-enable
    policy.enable_component(BadCog)
    assert has_exactly(1, GoodCog, widget.cogs)
    assert has_exactly(1, BadCog, widget.cogs)

    policy.disable_component('tests.test_policy.BadCog')
    assert has_exactly(1, GoodCog, widget.cogs)
    assert has_exactly(0, BadCog, widget.cogs)

    policy.enable_component('tests.test_policy.BadCog')
    assert has_exactly(1, GoodCog, widget.cogs)
    assert has_exactly(1, BadCog, widget.cogs)


def test_whitelist():
    clear_registry()
    from giblets import Component, ComponentManager, ExtensionPoint, ExtensionInterface, implements
    from giblets.policy import Whitelist
    
    class ICog(ExtensionInterface):
        pass

    class Widget(Component):
        cogs = ExtensionPoint(ICog)

    class GoodCog(Component):
        implements(ICog)

    class BadCog(Component):
        implements(ICog)

    
    policy = Whitelist()
    mgr = ComponentManager()
    mgr.restrict(policy)
    
    # first nothing specified, nothing should show up
    widget = Widget(mgr)
    assert has_exactly(0, GoodCog, widget.cogs)
    assert has_exactly(0, BadCog, widget.cogs)

    # enable only the GoodCog
    policy.enable_component(GoodCog)
    assert has_exactly(1, GoodCog, widget.cogs)
    assert has_exactly(0, BadCog, widget.cogs)

    # re-disable, should show nothing again
    policy.disable_component(GoodCog)
    assert has_exactly(0, GoodCog, widget.cogs)
    assert has_exactly(0, BadCog, widget.cogs)

    # try enable by name...
    policy.enable_component('tests.test_policy.GoodCog')
    assert has_exactly(1, GoodCog, widget.cogs)
    assert has_exactly(0, BadCog, widget.cogs)

    policy.disable_component('tests.test_policy.GoodCog')
    assert has_exactly(0, GoodCog, widget.cogs)
    assert has_exactly(0, BadCog, widget.cogs)


def test_patterns():
    clear_registry()
    from giblets import Component, ComponentManager, ExtensionPoint, ExtensionInterface, implements
    from giblets.policy import Patterns

    class ICog(ExtensionInterface):
        pass

    class Widget(Component):
        cogs = ExtensionPoint(ICog)

    class GoodCog(Component):
        implements(ICog)

    class BadCog(Component):
        implements(ICog)

    policy = Patterns()
    mgr = ComponentManager()
    mgr.restrict(policy)
    widget = Widget(mgr)

    # to start with, everything is disabled...
    assert has_exactly(0, GoodCog, widget.cogs)
    assert has_exactly(0, BadCog, widget.cogs)

    # easy to enable everything...
    policy.append_pattern('*', enable=True)
    assert has_exactly(1, GoodCog, widget.cogs)
    assert has_exactly(1, BadCog, widget.cogs)

    # now just disable the bad one
    pat = policy.build_pattern('tests.test_policy.BadCog', enable=False)
    policy.patterns.insert(0, pat)
    assert has_exactly(1, GoodCog, widget.cogs)
    assert has_exactly(0, BadCog, widget.cogs)
    
    # disable all the test core components...
    pat = policy.build_pattern('tests.test_policy.*', enable=False)
    policy.patterns.insert(0, pat)
    assert has_exactly(0, GoodCog, widget.cogs)
    assert has_exactly(0, BadCog, widget.cogs)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2009-2010 Luke Tucker
# All rights reserved.
#
# This software is licensed as described in the file COPYING, which
# you should have received as part of this distribution.
#
# Author: Luke Tucker <voxluci@gmail.com>
#

from setuptools import setup, find_packages

version = "0.2.1"

long_description = """
Giblets is a simple plugin system based on the component architecture of 
`Trac <http://trac.edgewall.org>`_.
In a nutshell, giblets allows you to declare interfaces and discover components
that implement them without coupling.

Giblets also includes plugin discovery based on file paths or entry points along with 
flexible means to manage which components are enabled or disabled in your application.

The full documenation for giblets is available at 
`http://ltucker.github.com/giblets <http://ltucker.github.com/giblets>`_.

Installation
=============

To install::

    $ pip install giblets

or::
    
    $ easy_install giblets

You can also install the current 
`development version <http://github.com/ltucker/giblets/tarball/master#egg=giblets-dev>`_ 
of giblets with ``pip install giblets==dev`` or ``easy_install giblets==dev`` 
"""

setup(
    name='giblets',
    version=version,
    description="A simple plugin system based on the component architecture of Trac.",
    long_description=long_description,
    license="BSD",
    author="Luke Tucker",
    author_email="voxluci@gmail.com",
    url="http://github.com/ltucker/giblets",
    install_requires=[
        "zope.interface",
    ],
    dependency_links=[
    ],
    packages=find_packages(),
    namespace_packages=[],
    include_package_data=True,
    test_suite = 'nose.collector',
    entry_points="""
    [giblets_load_from_entry_point_test]
    mod1 = tests.test_plugin_egg.eggmod1
    mod2 = tests.test_plugin_egg.eggmod2
    """,
    classifiers=[
      'Development Status :: 4 - Beta',
      'Intended Audience :: Developers',
      'License :: OSI Approved :: BSD License',
      'Topic :: Software Development'
    ],
    keywords = "plugins components",
)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2009-2010 Luke Tucker
# All rights reserved.
#
# This software is licensed as described in the file COPYING, which
# you should have received as part of this distribution.
#
# Author: Luke Tucker <voxluci@gmail.com>
#

from giblets import ExtensionInterface

class TestPathInterface(ExtensionInterface):
    pass

class TestEggInterface(ExtensionInterface):
    pass

def test_load_from_path():
    from giblets.core import ComponentManager, Component, ExtensionPoint
    from giblets.search import find_plugins_in_path
    
    class PluginFinder(Component):
        found_plugins = ExtensionPoint(TestPathInterface)

    mgr = ComponentManager()
    pf = PluginFinder(mgr)
    
    # to start with, nothing should be found    
    assert len(pf.found_plugins) == 0
    
    find_plugins_in_path('test_plugin_path')

    expected_plugins = ['TestPathPlugin1', 'TestPathPlugin2', 'TestPathPlugin3']
    got_plugins = set()

    assert len(pf.found_plugins) == len(expected_plugins)    
    for plugin in pf.found_plugins:
        plugin_name = plugin.__class__.__name__
        assert plugin_name in expected_plugins
        got_plugins.add(plugin_name)
    for plugin_name in expected_plugins:
        assert plugin_name in got_plugins
    
def test_load_from_entry_point():
    from giblets.core import ComponentManager, Component, ExtensionPoint
    from giblets.search import find_plugins_by_entry_point
    
    class PluginFinder(Component):
        found_plugins = ExtensionPoint(TestEggInterface)

    mgr = ComponentManager()
    pf = PluginFinder(mgr)
    
    # to start with, nothing should be found    
    assert len(pf.found_plugins) == 0
    
    find_plugins_by_entry_point('giblets_load_from_entry_point_test')

    expected_plugins = ['TestEggPlugin1', 'TestEggPlugin2', 'TestEggPlugin3']
    got_plugins = set()

    assert len(pf.found_plugins) == len(expected_plugins)    
    for plugin in pf.found_plugins:
        plugin_name = plugin.__class__.__name__
        assert plugin_name in expected_plugins
        got_plugins.add(plugin_name)
    for plugin_name in expected_plugins:
        assert plugin_name in got_plugins
    



 
def has_exactly(k, T, l):
    """
    test that the list l has exactly k members that are 
    instances of the type T
    """
    return sum(1 for x in l if isinstance(x, T)) == k

def clear_registry():
    from giblets.core import ComponentMeta
    ComponentMeta._registry = {}
    ComponentMeta._components = []
